@@ -26,7 +26,7 @@ def _get_arguments_from_request():
     assert issubclass(request_data['arguments'].__class__, dict) is True
 
     try:
-        return request_data['arguments'].items()
+        return request_data['arguments'].iteritems()
     except KeyError:
         raise ValueError("No arguments given (2)")
 
@@ -43,10 +43,11 @@ def job_submit(workflow_name, job_name):
     handler = mr.models.kv.handler.get(workflow, step.map_handler_name)
 
     arguments = _get_arguments_from_request()
+    arguments = list(arguments)
 
     remote_addr_header = mr.config.request.REMOTE_ADDR_HEADER
 
-    _IS_BLOCKING = (flask.request.args.get('asynchronous', 'false') == 'false')
+    is_blocking = (flask.request.args.get('blocking', 'true') == 'true')
 
     context = {
         'requester_ip': flask.request.environ[remote_addr_header]
@@ -60,15 +61,35 @@ def job_submit(workflow_name, job_name):
                             step, 
                             handler, 
                             arguments, 
-                            context)
+                            context, 
+                            is_blocking=is_blocking)
 
     request = message_parameters.request
+
+    _logger.debug("Pushing request [%s] with initial step [%s] and invocation [%s] for mapper "
+                  "[%s].", request, step, message_parameters.invocation, handler)
+
     response = {}
 
     try:
-        result = rr.process_request(
-                    message_parameters, 
-                    block=_IS_BLOCKING)
+        rr.push_request(message_parameters)
+
+        if is_blocking is True:
+            result = rr.block_for_result(message_parameters)
+
+            # Some result writers will only be used when blocking and others 
+            # when not blocking, and others in both. The latter will never
+            # have anything to return in the request-response, so it should
+            # always be None. In that case, we'll need to change it to an empty
+            # dictionary, since we like to return our results as dictionaries.
+            if result is None:
+                _logger.debug("The result writer returned None in blocking-"
+                              "mode. Kindly changing to an empty dictionary: "
+                              "%s", request)
+                result = {}
+        else:
+            result = None
+
     except Exception as e:
         _logger.exception("Request failed.")
 
@@ -80,9 +101,13 @@ def job_submit(workflow_name, job_name):
         exception_type = None
         exception_message = None
 
-        response['result'] = result
+        if is_blocking is True:
+            if issubclass(result.__class__, dict) is False:
+                raise ValueError("Result must be returned as a dictionary to "
+                                 "the blocking request-response: %s" % 
+                                 (request,))
 
-# TODO(dustin): Asynchronous requests fail when we exit too soon. Since we've confirmed that we can read it back immediately after writing it and we don't have a cache, we don't know why the message-handlers can't read it.
+        response['result'] = result
 
     raw_response = flask.jsonify(response)
     response = flask.make_response(raw_response)
@@ -93,6 +118,4 @@ def job_submit(workflow_name, job_name):
         response.headers['X-MR-EXCEPTION-TYPE'] = exception_type
         response.headers['X-MR-EXCEPTION-MESSAGE'] = exception_message
 
-    managed_workflow.cleanup_queue.add_request(request)
-
     return (response, code)
